unit ukohonen;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Grids, SDL_NTabEd, SDL_RChart, ComCtrls, Tabnotbk, StdCtrls, Buttons, ExtCtrls,
  SDL_kohonen, SDL_NumLab, SDL_NumIO, SDL_vector;

type
  TFrmSOM = class(TForm)
    Panel1: TPanel;
    BButExit: TBitBtn;
    NoteBk: TTabbedNotebook;
    RCKohMap: TRChart;
    TEData: TNTabEd;
    BButLoad: TBitBtn;
    OpenDialog1: TOpenDialog;
    BButTrainIt: TBitBtn;
    SBAlpha: TScrollBar;
    SBSteps: TScrollBar;
    NLAlpha: TNumLab;
    NLSteps: TNumLab;
    NIOysize: TNumIO;
    NIOxsize: TNumIO;
    Label1: TLabel;
    Label2: TLabel;
    NIONeighb: TNumIO;
    Label3: TLabel;
    BButStoreNet: TBitBtn;
    BButLoadNet: TBitBtn;
    BButContTrain: TBitBtn;
    SaveDialog1: TSaveDialog;
    KohMap1: TKohonen;
    RowVect: TVector;
    procedure BButExitClick(Sender: TObject);
    procedure BButLoadClick(Sender: TObject);
    procedure BButTrainItClick(Sender: TObject);
    procedure SBAlphaChange(Sender: TObject);
    procedure SBStepsChange(Sender: TObject);
    procedure BButStoreNetClick(Sender: TObject);
    procedure BButLoadNetClick(Sender: TObject);
    procedure BButContTrainClick(Sender: TObject);
    procedure KohMap1Feedback(Sender: TObject; PercentDone: Integer);
  private
    procedure ShowCurrentNetResponse;
  public
  end;

var
  FrmSOM: TFrmSOM;

implementation

{$R *.DFM}

uses
  SDL_math1, SDL_filesys, Uprogres;


const
  MaxSteps = 19;
  Steps : array[1..MaxSteps] of integer =
                             (10,15,20,30,50,75,
                              100,150,200,300,500,750,
                              1000,1500,2000,3000,5000,7500,10000);
  MaxPreDefColors = 16;
  PredefinedColors : array [0..MaxPreDefColors-1] of TColor =
              (clRed, clBlue, clLime, clYellow, clFuchsia,
               clBlack, clMaroon, clGreen, clOlive, clNavy,
               clPurple, clTeal, clGray, clSilver, clAqua,
               clWhite);



(******************************************************************************)
procedure TFrmSOM.BButExitClick(Sender: TObject);
(******************************************************************************)

begin
close;
end;


(******************************************************************************)
procedure TFrmSOM.BButLoadClick(Sender: TObject);
(******************************************************************************)

begin
NoteBk.PageIndex := 0;
OpenDialog1.Filter := 'Data Files|*.asc';
if OpenDialog1.Execute then
  begin
  TEData.SuppressPaint := true;
  TEData.Data.ImportASC (OpenDialog1.FileName);
  TEData.SuppressPaint := false;
  BButTrainit.Enabled := true;
  end;
end;


(******************************************************************************)
procedure TFrmSOM.ShowCurrentNetResponse;
(******************************************************************************)

var
  ix, iy : integer;
  dist   : double;
  i      : integer;

begin
RCKohMap.ClearGraf;
RowVect.NrOfElem := TEData.NrOfColumns;
for i:=1 to TEData.NrOfRows do
  begin
  TEData.Data.NumericData.CopyRowToVec (RowVect,i,1,TEData.NrOfColumns);
  KohMap1.ApplyIt (RowVect, Ix, Iy, Dist);
  RCKohMap.DataColor := PredefinedColors[TEData.RowAttrib[i] mod 16];
  RCKohMap.MarkAt (ix+0.6*random-0.3, iy+0.6*random-0.3, 11);
  end;
RCKohMap.SetRange (1, 0, 0, KohMap1.SizeX+1,KohMap1.SizeY+1);
RCKohMap.ShowGraf;
end;


(******************************************************************************)
procedure TFrmSOM.BButTrainItClick(Sender: TObject);
(******************************************************************************)


begin
BButContTrain.Enabled := false;
NoteBk.PageIndex := 1;
SBAlpha.Enabled := false;
SBSteps.Enabled := false;
KohMap1.TrainData := TEData.Data.NumericData;
KohMap1.StandardizeData;
KohMap1.SizeX := round(NIOxSize.Value);
KohMap1.SizeY := round(NIOySize.Value);
KohMap1.InitialAlpha := SBAlpha.Position/100;
KohMap1.NrOfTrnSteps := Steps[SBSteps.Position];
KohMap1.InitialNeighbors := round(NIONeighb.Value);
FrmProgress.Show;
Enabled := false;
KohMap1.TrainIt;
FrmProgress.Hide;
Enabled := true;
ShowCurrentNetResponse;
Refresh;
SBAlpha.Enabled := true;
SBSteps.Enabled := true;
end;

(******************************************************************************)
procedure TFrmSOM.SBAlphaChange(Sender: TObject);
(******************************************************************************)

begin
NLAlpha.Value := SBAlpha.Position/100;
end;

(******************************************************************************)
procedure TFrmSOM.SBStepsChange(Sender: TObject);
(******************************************************************************)

begin
NLSteps.Value := Steps[SBSteps.Position];
end;


(******************************************************************************)
procedure TFrmSOM.BButStoreNetClick(Sender: TObject);
(******************************************************************************)

begin
SaveDialog1.Filter := 'XML Files|*.xml';
if SaveDialog1.Execute then
  begin
  KohMap1.SaveAsXMLFile (StripExtension(SaveDialog1.Filename)+'.xml', '');
  end;
end;


(******************************************************************************)
procedure TFrmSOM.BButLoadNetClick(Sender: TObject);
(******************************************************************************)

var
  i : integer;

begin
OpenDialog1.Filter := 'XML Files|*.xml';
if OpenDialog1.Execute then
  begin
  NoteBk.PageIndex := 1;
  KohMap1.LoadFromXMLFile (OpenDialog1.Filename, '');
  NIOXSize.Value := KohMap1.SizeX;
  NIOYSize.Value := KohMap1.SizeY;
  NIONeighb.Value := KohMap1.InitialNeighbors;
  SBAlpha.Position := round(KohMap1.InitialAlpha*100);
  NLAlpha.Value := SBAlpha.Position/100;
  i := 1;
  while (Steps[i] < KohMap1.NrOfTrnSteps) and (i < MaxSteps) do
    inc (i);
  SBSteps.Position := i;
  NLSteps.Value := Steps[SBSteps.Position];
  ShowCurrentNetREsponse;
  BButContTrain.Enabled := true;
  end;
end;

(******************************************************************************)
procedure TFrmSOM.BButContTrainClick(Sender: TObject);
(******************************************************************************)

begin
FrmProgress.Show;
Enabled := false;
KohMap1.TrainData := TEData.Data.NumericData;
KohMap1.StandardizeData;
KohMap1.ContinueTraining;
FrmProgress.Hide;
Enabled := true;
ShowCurrentNetResponse;
SBAlpha.Enabled := true;
SBSteps.Enabled := true;
Show;
end;

(******************************************************************************)
procedure TFrmSOM.KohMap1Feedback(Sender: TObject; PercentDone: Integer);
(******************************************************************************)

begin      { here the display of the progress takes place }
FrmProgress.Meter.Value := PercentDone;
if FrmProgress.CBShowTraining.Checked then
  ShowCurrentNetResponse;
Application.ProcessMessages;
end;


end.
